@@ -50,6 +50,10 @@ int main(int argc, char* argv[])
     cxxtools::Arg<bool> jsonhttp(argc, argv, 'J');
     cxxtools::Arg<unsigned short> port(argc, argv, 'p', binary ? 7003 : json ? 7004 : 7002);
 
+    // Normally we would define just one rpc client for the protocol we use but
+    // here we want to demonstrate, that it is just up to the client, which protocol
+    // is used for the remote call.
+
     // define a xlmrpc client
     cxxtools::xmlrpc::HttpClient xmlrpcClient(ip, port, "/xmlrpc");
     // and a binary rpc client
@@ -60,13 +64,16 @@ int main(int argc, char* argv[])
     // and a http json rpc client
     cxxtools::json::HttpClient httpJsonClient(ip, port,"/jsonrpc");
 
-    // define remote procedure with dobule return value and a double and a std::string parameter:
-    // Note: We send the second parameter as a string since it is converted from the server anyway,
-    cxxtools::RemoteProcedure<double, double, std::string> add(
+    // now se welect the client depending on the command line flags
+    cxxtools::RemoteClient& theClient =
         binary   ? static_cast<cxxtools::RemoteClient&>(binaryClient) :
         json     ? static_cast<cxxtools::RemoteClient&>(jsonClient) :
         jsonhttp ? static_cast<cxxtools::RemoteClient&>(httpJsonClient) :
-                   static_cast<cxxtools::RemoteClient&>(xmlrpcClient), "add");
+                   static_cast<cxxtools::RemoteClient&>(xmlrpcClient);
+
+    // define remote procedure with dobule return value and a double and a std::string parameter:
+    // Note: We send the second parameter as a string since it is converted by the server anyway.
+    cxxtools::RemoteProcedure<double, double, std::string> add(theClient, "add");
 
     double sum = 0;
     for (int a = 1; a < argc; ++a)
