@@ -114,9 +114,11 @@ class CXXTOOLS_HTTP_API Client : private NonCopyable
 
         void setSelector(SelectorBase& selector);
 
+        SelectorBase* selector();
+
         // Executes the underlying selector until a event occures or the
         // specified timeout is reached.
-        void wait(std::size_t msecs);
+        bool wait(std::size_t msecs);
 
         // Returns the underlying stream, where the reply may be read from.
         std::istream& in();
