@@ -53,9 +53,11 @@ class CXXTOOLS_HTTP_API Server : private cxxtools::NonCopyable
     public:
         explicit Server(EventLoopBase& eventLoop);
         Server(EventLoopBase& eventLoop, const std::string& ip, unsigned short int port, int backlog = 64);
+        Server(EventLoopBase& eventLoop, unsigned short int port, int backlog = 64);
         ~Server();
 
         void listen(const std::string& ip, unsigned short int port, int backlog = 64);
+        void listen(unsigned short int port, int backlog = 64);
 
         void addService(const std::string& url, Service& service);
         void addService(const Regex& url, Service& service);
