@@ -29,9 +29,10 @@
 #ifndef CXXTOOLS_MD5STREAM_H
 #define CXXTOOLS_MD5STREAM_H
 
-#include <cxxtools/md5.h>
 #include <iostream>
 
+struct cxxtools_MD5_CTX;
+
 namespace cxxtools
 {
 
@@ -39,13 +40,14 @@ class Md5streambuf : public std::streambuf
 {
   public:
     Md5streambuf();
+    ~Md5streambuf();
 
     void getDigest(unsigned char digest[16]);
 
   private:
     static const unsigned int bufsize = 64;
     char buffer[bufsize];
-    cxxtools_MD5_CTX context;
+    cxxtools_MD5_CTX* context;
     unsigned char digest[16];
 
     std::streambuf::int_type overflow(std::streambuf::int_type ch);
