@@ -30,36 +30,38 @@
 #define CXXTOOLS_PROPERTIES_H
 
 #include <string>
-#include <iosfwd>
 #include <map>
+#include <cxxtools/char.h>
+#include <cxxtools/textstream.h>
+#include <cxxtools/serializationerror.h>
 
 namespace cxxtools
 {
   class Properties
   {
-      typedef std::map<std::string, std::string> ValuesType;
+      typedef std::map<String, String> ValuesType;
       ValuesType values;
 
     public:
       explicit Properties(const std::string& filename);
       explicit Properties(std::istream& in);
 
-      void setValue(const std::string& key, const std::string& value)
+      void setValue(const String& key, const String& value)
       {
         values[key] = value;
       }
 
-      void removeValue(const std::string& key)
+      void removeValue(const String& key)
       {
         values.erase(key);
       }
 
-      bool hasValue(const std::string& key) const
+      bool hasValue(const String& key) const
       {
         return values.find(key) != values.end();
       }
 
-      std::string getValue(const std::string& key, const std::string& def = std::string()) const
+      String getValue(const String& key, const String& def = String()) const
       {
         ValuesType::const_iterator it = values.find(key);
         return it == values.end() ? def : it->second;
@@ -76,15 +78,15 @@ namespace cxxtools
       }
 
       template <typename OutputIterator>
-      void getKeys(const std::string& praefix, OutputIterator oi) const
+      void getKeys(const String& praefix, OutputIterator oi) const
       {
         for (ValuesType::const_iterator it = values.begin(); it != values.end(); ++it)
         {
-          std::string key = it->first;
+          String key = it->first;
           if (key.size() > praefix.size()
-            && key.at(praefix.size()) == '.'
+            && key.at(praefix.size()) == L'.'
             && key.compare(0, praefix.size(), praefix) == 0
-            && key.find('.', praefix.size() + 2) == std::string::npos)
+            && key.find(L'.', praefix.size() + 2) == String::npos)
           {
             *oi = key.substr(praefix.size() + 1);
             ++oi;
@@ -93,15 +95,15 @@ namespace cxxtools
       }
 
       template <typename OutputIterator>
-      void getKeysLong(const std::string& praefix, OutputIterator oi) const
+      void getKeysLong(const String& praefix, OutputIterator oi) const
       {
         for (ValuesType::const_iterator it = values.begin(); it != values.end(); ++it)
         {
-          std::string key = it->first;
+          String key = it->first;
           if (key.size() > praefix.size()
             && key.at(praefix.size()) == '.'
             && key.compare(0, praefix.size(), praefix) == 0
-            && key.find('.', praefix.size() + 2) == std::string::npos)
+            && key.find('.', praefix.size() + 2) == String::npos)
           {
             *oi = key;
             ++oi;
@@ -117,37 +119,55 @@ namespace cxxtools
       class Event
       {
         public:
+          virtual ~Event()  { }
           // return true, if parser should stop
-          virtual bool onKeyPart(const std::string& key) = 0;
-          virtual bool onKey(const std::string& key) = 0;
-          virtual bool onValue(const std::string& value) = 0;
+          virtual bool onKeyPart(const String& key) = 0;
+          virtual bool onKey(const String& key) = 0;
+          virtual bool onValue(const String& value) = 0;
       };
 
     private:
       Event& event;
-      std::string key;
-      std::string keypart;
-      std::string value;
+      String key;
+      String keypart;
+      String value;
+      Char::value_type unicode;
+      unsigned unicodeCount;
+      unsigned lineNo;
+
       enum {
         state_0,
         state_key,
+        state_key_esc,
+        state_key_unicode,
         state_key_sp,
         state_value,
         state_value_esc,
+        state_unicode,
         state_comment
       } state;
 
     public:
       PropertiesParser(Event& event_)
         : event(event_),
+          lineNo(1),
           state(state_0)
         { }
 
-      void parse(std::istream& in);
-      bool parse(char ch);
+      void parse(TextIStream& in);
+      void parse(std::istream& in, TextCodec<Char, char>* codec = 0);
+      bool parse(Char ch);
       void end();
   };
 
+  class PropertiesParserError : public SerializationError
+  {
+    public:
+      explicit PropertiesParserError(const std::string& msg)
+        : SerializationError(msg)
+        { }
+      PropertiesParserError(const std::string& msg, unsigned lineNo);
+  };
 }
 
 #endif // CXXTOOLS_PROPERTIES_H
