@@ -101,7 +101,7 @@ namespace unit {
             if( ! ((value1) == (value2)) ) \
             { \
                 std::ostringstream _cxxtools_msg; \
-                _cxxtools_msg << "not equal: value1 (" #value1 ")=<" << value1 << "> value2 (" #value2 ")=<" << value2 << '>'; \
+                _cxxtools_msg << "not equal:\n\tvalue1 (" #value1 ")=\n\t\t<" << value1 << ">\n\tvalue2 (" #value2 ")=\n\t\t<" << value2 << '>'; \
                 throw cxxtools::unit::Assertion(_cxxtools_msg.str(), CXXTOOLS_SOURCEINFO); \
             } \
         } while (false)
@@ -136,6 +136,10 @@ namespace unit {
                     std::string("unexpected exception of type ") + typeid(e).name() + ": " + e.what(), \
                     CXXTOOLS_SOURCEINFO); \
             } \
+            catch(const cxxtools::unit::Assertion&) \
+            { \
+                throw; \
+            } \
             catch(...) \
             { \
                 throw cxxtools::unit::Assertion("unexpected exception." , CXXTOOLS_SOURCEINFO); \
