@@ -160,6 +160,19 @@ const SerializationInfo& SerializationInfo::getMember(const std::string& name) c
 }
 
 
+const SerializationInfo& SerializationInfo::getMember(unsigned idx) const
+{
+    if (idx >= _nodes.size())
+    {
+        std::ostringstream msg;
+        msg << "requested member index " << idx << " exceeds number of members " << _nodes.size();
+        throw std::range_error(msg.str());
+    }
+
+    return _nodes[idx];
+}
+
+
 const SerializationInfo* SerializationInfo::findMember(const std::string& name) const
 {
     Nodes::const_iterator it = _nodes.begin();
@@ -299,36 +312,48 @@ void SerializationInfo::swap(SerializationInfo& si)
 
 void SerializationInfo::dump(std::ostream& out, const std::string& praefix) const
 {
-    out << praefix << "type = " << (_t == t_none ? "none" :
-                                    _t == t_string ? "string" :
-                                    _t == t_string8 ? "string8" :
-                                    _t == t_char ? "char" :
-                                    _t == t_bool ? "bool" :
-                                    _t == t_int ? "int" :
-                                    _t == t_uint ? "uint" :
-                                    _t == t_float ? "float" : "?")
-        << praefix << "value = ";
+    if (!_name.empty())
+        out << praefix << "name = \"" << _name << "\"\n";
 
-    switch (_t)
+    if (_t != t_none)
     {
-        case t_none:    out << '-'; break;
-        case t_string:  out << '"' << _String().narrow() << '"'; break;
-        case t_string8: out << '"' << _String8() << '"'; break;
-        case t_char:    out << '\'' << _u._c << '\''; break;
-        case t_bool:    out << _u._b; break;
-        case t_int:     out << _u._i; break;
-        case t_uint:    out << _u._u; break;
-        case t_float:   out << _u._f; break;
+        out << praefix << "type = " << (_t == t_none ? "none" :
+                                        _t == t_string ? "string" :
+                                        _t == t_string8 ? "string8" :
+                                        _t == t_char ? "char" :
+                                        _t == t_bool ? "bool" :
+                                        _t == t_int ? "int" :
+                                        _t == t_uint ? "uint" :
+                                        _t == t_float ? "float" : "?") << '\n';
+
+        out << praefix << "value = ";
+
+        switch (_t)
+        {
+            case t_none:    out << '-'; break;
+            case t_string:  out << '"' << _String().narrow() << '"'; break;
+            case t_string8: out << '"' << _String8() << '"'; break;
+            case t_char:    out << '\'' << _u._c << '\''; break;
+            case t_bool:    out << _u._b; break;
+            case t_int:     out << _u._i; break;
+            case t_uint:    out << _u._u; break;
+            case t_float:   out << _u._f; break;
+        }
+
+        out << '\n';
     }
 
-    out << "\n"
-        << praefix << "name = " << _name << "\n"
-        << praefix << "type = " << _type << "\n"
-        << praefix << "nodes.size = " << _nodes.size() << "\n";
-
-    std::string p = praefix + '\t';
-    for (Nodes::const_iterator it = _nodes.begin(); it != _nodes.end(); ++it)
-        it->dump(out, p);
+    if (!_type.empty())
+        out << praefix << "typeName = " << _type << '\n';
+    if (!_nodes.empty())
+    {
+        std::string p = praefix + '\t';
+        for (std::vector<SerializationInfo>::size_type n = 0; n < _nodes.size(); ++n)
+        {
+            out << praefix << "node[" << n << "]\n";
+            _nodes[n].dump(out, p);
+        }
+    }
 }
 
 void SerializationInfo::_releaseValue()
@@ -538,7 +563,7 @@ wchar_t SerializationInfo::_getWChar() const
     switch (_t)
     {
         case t_none:    return L'\0';
-        case t_string:  return _String().empty() ? Char(0) : _String()[0];
+        case t_string:  return _String().empty() ? L'\0' : _String()[0].toWchar();
         case t_string8: return _String8().empty() ? '\0' : _String8()[0];
         case t_char:    return _u._c;
         case t_bool:    return _u._b;
