@@ -28,19 +28,13 @@
  */
 
 #include <cxxtools/string.h>
-#include <cxxtools/textstream.h>
 #include <cxxtools/utf8codec.h>
 #include <iostream>
 #include <algorithm>
 
-#ifndef INLINE
-#define INLINE
-#endif
-
 namespace std
 {
 
-INLINE
 void basic_string<cxxtools::Char>::resize(size_t n, cxxtools::Char ch)
 {
     size_type size = this->size();
@@ -53,7 +47,6 @@ void basic_string<cxxtools::Char>::resize(size_t n, cxxtools::Char ch)
 }
 
 
-INLINE
 void basic_string<cxxtools::Char>::reserve(size_t n)
 {
     if (capacity() < n)
@@ -78,7 +71,6 @@ void basic_string<cxxtools::Char>::reserve(size_t n)
 }
 
 
-INLINE
 void basic_string<cxxtools::Char>::privreserve(size_t n)
 {
     if (capacity() < n)
@@ -91,7 +83,6 @@ void basic_string<cxxtools::Char>::privreserve(size_t n)
 }
 
 
-INLINE
 void basic_string<cxxtools::Char>::swap(basic_string& str)
 {
     if (isShortString())
@@ -129,7 +120,6 @@ void basic_string<cxxtools::Char>::swap(basic_string& str)
 
 
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::copy(cxxtools::Char* a, size_type n, size_type pos) const
 {
@@ -147,7 +137,6 @@ basic_string<cxxtools::Char>::copy(cxxtools::Char* a, size_type n, size_type pos
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const basic_string<cxxtools::Char>& str)
 {
     // self-assignment check
@@ -166,7 +155,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const basic_s
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const std::string& str)
 {
     size_type len = str.length();
@@ -182,7 +170,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const std::st
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const std::string& str, size_type pos, size_type len)
 {
     privreserve(len);
@@ -197,7 +184,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const std::st
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const wchar_t* str)
 {
     size_type length = 0;
@@ -209,7 +195,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const wchar_t
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const wchar_t* str, size_type length)
 {
     privreserve(length);
@@ -225,7 +210,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const wchar_t
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const char* str)
 {
     size_type length = 0;
@@ -237,7 +221,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const char* s
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const char* str, size_type length)
 {
     privreserve(length);
@@ -253,7 +236,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const char* s
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const cxxtools::Char* str, size_type length)
 {
     // self-assignment check
@@ -269,7 +251,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(const cxxtool
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(size_type n, cxxtools::Char ch)
 {
     privreserve(n);
@@ -284,7 +265,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::assign(size_type n,
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::append(const cxxtools::Char* str, size_type n)
 {
     size_type l = length();
@@ -296,7 +276,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::append(const cxxtool
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::append(size_type n, cxxtools::Char ch)
 {
     size_type l = length();
@@ -310,7 +289,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::append(size_type n,
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::insert(size_type pos, const cxxtools::Char* str, size_type n)
 {
     size_type l = length();
@@ -324,7 +302,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::insert(size_type pos
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::insert(size_type pos, size_type n, cxxtools::Char ch)
 {
     size_type l = length();
@@ -339,7 +316,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::insert(size_type pos
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::erase(size_type pos, size_type n)
 {
     cxxtools::Char* p = privdata_rw();
@@ -353,7 +329,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::erase(size_type pos,
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::replace(size_type pos, size_type n, const cxxtools::Char* str, size_type n2)
 {
     cxxtools::Char* p;
@@ -375,7 +350,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::replace(size_type po
 }
 
 
-INLINE
 basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::replace(size_type pos, size_type n, size_type n2, cxxtools::Char ch)
 {
     cxxtools::Char* p;
@@ -399,7 +373,6 @@ basic_string<cxxtools::Char>& basic_string<cxxtools::Char>::replace(size_type po
 }
 
 
-INLINE
 int basic_string<cxxtools::Char>::compare(const basic_string& str) const
 {
     const size_type size = this->size();
@@ -417,7 +390,6 @@ int basic_string<cxxtools::Char>::compare(const basic_string& str) const
 }
 
 
-INLINE
 int basic_string<cxxtools::Char>::compare(const char* str) const
 {
     size_type size = length();
@@ -434,7 +406,6 @@ int basic_string<cxxtools::Char>::compare(const char* str) const
 }
 
 
-INLINE
 int basic_string<cxxtools::Char>::compare(const char* str, size_type len) const
 {
     size_type size = length();
@@ -451,7 +422,6 @@ int basic_string<cxxtools::Char>::compare(const char* str, size_type len) const
 }
 
 
-INLINE
 int basic_string<cxxtools::Char>::compare(const cxxtools::Char* str, size_type osize) const
 {
     const size_type size = this->size();
@@ -468,11 +438,10 @@ int basic_string<cxxtools::Char>::compare(const cxxtools::Char* str, size_type o
 }
 
 
-INLINE
 int basic_string<cxxtools::Char>::compare(const wchar_t* str) const
 {
     const cxxtools::Char* self = privdata_ro();
-    while(*self && *str)
+    while(self->toWchar() != L'\0' && *str != L'\0')
     {
         if( *self != *str )
             return *self < cxxtools::Char(*str) ? -1 : +1;
@@ -481,11 +450,10 @@ int basic_string<cxxtools::Char>::compare(const wchar_t* str) const
         ++str;
     }
 
-    return static_cast<int>( *self - cxxtools::Char(*str) );
+    return static_cast<int>(self->value()) - static_cast<int>(*str);
 }
 
 
-INLINE
 int basic_string<cxxtools::Char>::compare(const wchar_t* str, size_type n) const
 {
     const cxxtools::Char* self = privdata_ro();
@@ -497,11 +465,10 @@ int basic_string<cxxtools::Char>::compare(const wchar_t* str, size_type n) const
         ++self;
     }
 
-    return *self ? 1 : nn < n ? -1 : 0;
+    return self->value() != 0 ? 1 : nn < n ? -1 : 0;
 }
 
 
-INLINE
 int basic_string<cxxtools::Char>::compare(size_type pos, size_type n, const cxxtools::Char* str, size_type n2) const
 {
     const size_type size = n;
@@ -521,7 +488,6 @@ int basic_string<cxxtools::Char>::compare(size_type pos, size_type n, const cxxt
 }
 
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find(const cxxtools::Char* token, size_type pos, size_type n) const
 {
@@ -538,7 +504,6 @@ basic_string<cxxtools::Char>::find(const cxxtools::Char* token, size_type pos, s
 }
 
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find(cxxtools::Char ch, size_type pos) const
 {
@@ -559,7 +524,6 @@ basic_string<cxxtools::Char>::find(cxxtools::Char ch, size_type pos) const
 }
 
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::rfind(const cxxtools::Char* token, size_type pos, size_type n) const
 {
@@ -582,7 +546,6 @@ basic_string<cxxtools::Char>::rfind(const cxxtools::Char* token, size_type pos,
 }
 
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::rfind(cxxtools::Char ch, size_type pos) const
 {
@@ -603,7 +566,6 @@ basic_string<cxxtools::Char>::rfind(cxxtools::Char ch, size_type pos) const
     return npos;
 }
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find_first_of(const cxxtools::Char* s, size_type pos, size_type n) const
 {
@@ -619,7 +581,6 @@ basic_string<cxxtools::Char>::find_first_of(const cxxtools::Char* s, size_type p
     return npos;
 }
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find_last_of(const cxxtools::Char* s, size_type pos, size_type n) const
 {
@@ -645,7 +606,6 @@ basic_string<cxxtools::Char>::find_last_of(const cxxtools::Char* s, size_type po
     return npos;
 }
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find_first_not_of(const cxxtools::Char* tok, size_type pos, size_type n) const
 {
@@ -658,7 +618,6 @@ basic_string<cxxtools::Char>::find_first_not_of(const cxxtools::Char* tok, size_
     return npos;
 }
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find_first_not_of(const cxxtools::Char ch, size_type pos) const
 {
@@ -674,7 +633,6 @@ basic_string<cxxtools::Char>::find_first_not_of(const cxxtools::Char ch, size_ty
 }
 
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find_last_not_of(const cxxtools::Char* tok, size_type pos, size_type n) const
 {
@@ -696,7 +654,6 @@ basic_string<cxxtools::Char>::find_last_not_of(const cxxtools::Char* tok, size_t
     return npos;
 }
 
-INLINE
 basic_string<cxxtools::Char>::size_type
 basic_string<cxxtools::Char>::find_last_not_of(cxxtools::Char ch, size_type pos) const
 {
@@ -717,7 +674,6 @@ basic_string<cxxtools::Char>::find_last_not_of(cxxtools::Char ch, size_type pos)
 }
 
 
-INLINE
 std::string basic_string<cxxtools::Char>::narrow(char dfault) const
 {
     std::string ret;
@@ -733,7 +689,6 @@ std::string basic_string<cxxtools::Char>::narrow(char dfault) const
 }
 
 
-INLINE
 basic_string<cxxtools::Char> basic_string<cxxtools::Char>::widen(const char* str)
 {
     std::basic_string<cxxtools::Char> ret;
@@ -748,7 +703,6 @@ basic_string<cxxtools::Char> basic_string<cxxtools::Char>::widen(const char* str
 }
 
 
-INLINE
 basic_string<cxxtools::Char> basic_string<cxxtools::Char>::widen(const std::string& str)
 {
     std::basic_string<cxxtools::Char> ret;
@@ -763,11 +717,35 @@ basic_string<cxxtools::Char> basic_string<cxxtools::Char>::widen(const std::stri
 }
 
 
-INLINE
 ostream& operator<< (ostream& out, const basic_string<cxxtools::Char>& str)
 {
-    cxxtools::TextOStream tout(out, new cxxtools::Utf8Codec());
-    tout << str;
+    cxxtools::Utf8Codec codec;
+    char to[64];
+    cxxtools::MBState state;
+
+    cxxtools::Utf8Codec::result r;
+    const cxxtools::Char* from = str.data();
+    cxxtools::String::size_type size = str.size();
+
+    do{
+        const cxxtools::Char* from_next;
+
+        char* to_next = to;
+        r = codec.out(state, from, from + size, from_next, to, to + sizeof(to), to_next);
+
+        if (r == cxxtools::Utf8Codec::error)
+        {
+            out.setstate(std::ios::failbit);
+            break;
+        }
+
+        out.write(to, to_next - to);
+
+        size -= (from_next - from);
+        from = from_next;
+
+    } while (out.good() && r == cxxtools::Utf8Codec::partial);
+
     return out;
 }
 
