@@ -27,8 +27,10 @@
  */
 #include "threadimpl.h"
 #include "cxxtools/systemerror.h"
+#include "cxxtools/timespan.h"
 #include <errno.h>
 #include <signal.h>
+#include <unistd.h>
 #include <iostream>
 
 extern "C"
@@ -61,14 +63,14 @@ namespace cxxtools
 
 void ThreadImpl::detach()
 {
+    _detached = true;
+
     if( _id )
     {
         int ret = pthread_detach(_id);
         if(ret != 0)
             throw SystemError("pthread_detach");
     }
-
-    _detached = true;
 }
 
 
@@ -87,17 +89,31 @@ void ThreadImpl::start()
     pthread_attr_init(&attrs);
     //pthread_attr_setinheritsched(&attrs, PTHREAD_INHERIT_SCHED);
 
-    if(stacksize > 0)
+    if (stacksize > 0)
         pthread_attr_setstacksize(&attrs ,stacksize);
 
-    int ret = pthread_create(&_id, &attrs, thread_entry, this);
-    pthread_attr_destroy(&attrs);
-
-    if(ret != 0)
-        throw SystemError("pthread_create");
-
     if (_detached)
-        detach();
+    {
+        pthread_t id;
+        int ret = pthread_create(&id, &attrs, thread_entry, this);
+        pthread_attr_destroy(&attrs);
+
+        if (ret != 0)
+            throw SystemError("pthread_create");
+
+        ret = pthread_detach(id);
+        if (ret != 0)
+            throw SystemError("pthread_detach");
+    }
+    else
+    {
+        int ret = pthread_create(&_id, &attrs, thread_entry, this);
+        pthread_attr_destroy(&attrs);
+
+        if (ret != 0)
+            throw SystemError("pthread_create");
+    }
+
 }
 
 
@@ -106,7 +122,7 @@ void ThreadImpl::join()
     void* threadRet = 0;
     int ret = pthread_join(_id, &threadRet);
 
-    if(ret != 0)
+    if (ret != 0)
         throw SystemError("pthread_join");
 }
 
@@ -118,4 +134,28 @@ void ThreadImpl::terminate()
         throw SystemError("pthread_kill");
 }
 
+void ThreadImpl::sleep(unsigned int ms)
+{
+    Timespan ts = Timespan::gettimeofday();
+
+    useconds_t us = static_cast<useconds_t>(ms) * 1000;
+
+    if (usleep(us) == -1 && errno == EINTR)
+    {
+        ts = Timespan(ts.totalUSecs() + ms * 1000);
+
+        do
+        {
+            Timespan ts2 = Timespan::gettimeofday();
+
+            if (ts2.totalUSecs() >= ts.totalUSecs())
+                break;
+
+            us = ts.totalUSecs() - ts2.totalUSecs();
+
+        } while (usleep(us) == -1 && errno == EINTR);
+    }
+}
+
+
 }
