@@ -40,7 +40,7 @@ namespace
   struct Ent
   {
     const wchar_t* entity;
-    uint32_t charValue;
+    Char::value_type charValue;
   };
 
   static const Ent ent[] = {
@@ -302,253 +302,7 @@ namespace
     { L"amp", 0x0026 },
     { L"apos", 0x0027 },
     { L"lt", 0x003C },
-    { L"gt", 0x003E },
-    { L"nbsp", 0x00A0 },
-    { L"iexcl", 0x00A1 },
-    { L"cent", 0x00A2 },
-    { L"pound", 0x00A3 },
-    { L"curren", 0x00A4 },
-    { L"yen", 0x00A5 },
-    { L"brvbar", 0x00A6 },
-    { L"sect", 0x00A7 },
-    { L"uml", 0x00A8 },
-    { L"copy", 0x00A9 },
-    { L"ordf", 0x00AA },
-    { L"laquo", 0x00AB },
-    { L"not", 0x00AC },
-    { L"shy", 0x00AD },
-    { L"reg", 0x00AE },
-    { L"macr", 0x00AF },
-    { L"deg", 0x00B0 },
-    { L"plusmn", 0x00B1 },
-    { L"sup2", 0x00B2 },
-    { L"sup3", 0x00B3 },
-    { L"acute", 0x00B4 },
-    { L"micro", 0x00B5 },
-    { L"para", 0x00B6 },
-    { L"middot", 0x00B7 },
-    { L"cedil", 0x00B8 },
-    { L"sup1", 0x00B9 },
-    { L"ordm", 0x00BA },
-    { L"raquo", 0x00BB },
-    { L"frac14", 0x00BC },
-    { L"frac12", 0x00BD },
-    { L"frac34", 0x00BE },
-    { L"iquest", 0x00BF },
-    { L"Agrave", 0x00C0 },
-    { L"Acirc", 0x00C2 },
-    { L"Atilde", 0x00C3 },
-    { L"Auml", 0x00C4 },
-    { L"Aring", 0x00C5 },
-    { L"AElig", 0x00C6 },
-    { L"Ccedil", 0x00C7 },
-    { L"Egrave", 0x00C8 },
-    { L"Eacute", 0x00C9 },
-    { L"Ecirc", 0x00CA },
-    { L"Euml", 0x00CB },
-    { L"Igrave", 0x00CC },
-    { L"Iacute", 0x00CD },
-    { L"Icirc", 0x00CE },
-    { L"Iuml", 0x00CF },
-    { L"ETH", 0x00D0 },
-    { L"Ntilde", 0x00D1 },
-    { L"Ograve", 0x00D2 },
-    { L"Oacute", 0x00D3 },
-    { L"Ocirc", 0x00D4 },
-    { L"Otilde", 0x00D5 },
-    { L"Ouml", 0x00D6 },
-    { L"times", 0x00D7 },
-    { L"Oslash", 0x00D8 },
-    { L"Ugrave", 0x00D9 },
-    { L"Uacute", 0x00DA },
-    { L"Ucirc", 0x00DB },
-    { L"Uuml", 0x00DC },
-    { L"Yacute", 0x00DD },
-    { L"THORN", 0x00DE },
-    { L"szlig", 0x00DF },
-    { L"agrave", 0x00E0 },
-    { L"acirc", 0x00E2 },
-    { L"atilde", 0x00E3 },
-    { L"auml", 0x00E4 },
-    { L"aring", 0x00E5 },
-    { L"aelig", 0x00E6 },
-    { L"ccedil", 0x00E7 },
-    { L"egrave", 0x00E8 },
-    { L"eacute", 0x00E9 },
-    { L"ecirc", 0x00EA },
-    { L"euml", 0x00EB },
-    { L"igrave", 0x00EC },
-    { L"iacute", 0x00ED },
-    { L"icirc", 0x00EE },
-    { L"iuml", 0x00EF },
-    { L"eth", 0x00F0 },
-    { L"ntilde", 0x00F1 },
-    { L"ograve", 0x00F2 },
-    { L"oacute", 0x00F3 },
-    { L"ocirc", 0x00F4 },
-    { L"otilde", 0x00F5 },
-    { L"ouml", 0x00F6 },
-    { L"divide", 0x00F7 },
-    { L"oslash", 0x00F8 },
-    { L"ugrave", 0x00F9 },
-    { L"uacute", 0x00FA },
-    { L"ucirc", 0x00FB },
-    { L"uuml", 0x00FC },
-    { L"yacute", 0x00FD },
-    { L"thorn", 0x00FE },
-    { L"yuml", 0x00FF },
-    { L"OElig", 0x0152 },
-    { L"oelig", 0x0153 },
-    { L"Scaron", 0x0160 },
-    { L"scaron", 0x0161 },
-    { L"Yuml", 0x0178 },
-    { L"fnof", 0x0192 },
-    { L"circ", 0x02C6 },
-    { L"tilde", 0x02DC },
-    { L"Alpha", 0x0391 },
-    { L"Beta", 0x0392 },
-    { L"Gamma", 0x0393 },
-    { L"Delta", 0x0394 },
-    { L"Epsilon", 0x0395 },
-    { L"Zeta", 0x0396 },
-    { L"Eta", 0x0397 },
-    { L"Theta", 0x0398 },
-    { L"Iota", 0x0399 },
-    { L"Kappa", 0x039A },
-    { L"Lambda", 0x039B },
-    { L"Mu", 0x039C },
-    { L"Nu", 0x039D },
-    { L"Xi", 0x039E },
-    { L"Omicron", 0x039F },
-    { L"Pi", 0x03A0 },
-    { L"Rho", 0x03A1 },
-    { L"Sigma", 0x03A3 },
-    { L"Tau", 0x03A4 },
-    { L"Upsilon", 0x03A5 },
-    { L"Phi", 0x03A6 },
-    { L"Chi", 0x03A7 },
-    { L"Psi", 0x03A8 },
-    { L"Omega", 0x03A9 },
-    { L"alpha", 0x03B1 },
-    { L"beta", 0x03B2 },
-    { L"gamma", 0x03B3 },
-    { L"delta", 0x03B4 },
-    { L"epsilon", 0x03B5 },
-    { L"zeta", 0x03B6 },
-    { L"eta", 0x03B7 },
-    { L"theta", 0x03B8 },
-    { L"iota", 0x03B9 },
-    { L"kappa", 0x03BA },
-    { L"lambda", 0x03BB },
-    { L"mu", 0x03BC },
-    { L"nu", 0x03BD },
-    { L"xi", 0x03BE },
-    { L"omicron", 0x03BF },
-    { L"pi", 0x03C0 },
-    { L"rho", 0x03C1 },
-    { L"sigmaf", 0x03C2 },
-    { L"sigma", 0x03C3 },
-    { L"tau", 0x03C4 },
-    { L"upsilon", 0x03C5 },
-    { L"phi", 0x03C6 },
-    { L"chi", 0x03C7 },
-    { L"psi", 0x03C8 },
-    { L"omega", 0x03C9 },
-    { L"thetasym", 0x03D1 },
-    { L"upsih", 0x03D2 },
-    { L"piv", 0x03D6 },
-    { L"ensp", 0x2002 },
-    { L"emsp", 0x2003 },
-    { L"thinsp", 0x2009 },
-    { L"zwnj", 0x200C },
-    { L"zwj", 0x200D },
-    { L"lrm", 0x200E },
-    { L"rlm", 0x200F },
-    { L"ndash", 0x2013 },
-    { L"mdash", 0x2014 },
-    { L"lsquo", 0x2018 },
-    { L"rsquo", 0x2019 },
-    { L"sbquo", 0x201A },
-    { L"ldquo", 0x201C },
-    { L"rdquo", 0x201D },
-    { L"bdquo", 0x201E },
-    { L"dagger", 0x2020 },
-    { L"Dagger", 0x2021 },
-    { L"bull", 0x2022 },
-    { L"hellip", 0x2026 },
-    { L"permil", 0x2030 },
-    { L"prime", 0x2032 },
-    { L"Prime", 0x2033 },
-    { L"lsaquo", 0x2039 },
-    { L"rsaquo", 0x203A },
-    { L"oline", 0x203E },
-    { L"frasl", 0x2044 },
-    { L"euro", 0x20AC },
-    { L"image", 0x2111 },
-    { L"weierp", 0x2118 },
-    { L"real", 0x211C },
-    { L"trade", 0x2122 },
-    { L"alefsym", 0x2135 },
-    { L"larr", 0x2190 },
-    { L"uarr", 0x2191 },
-    { L"rarr", 0x2192 },
-    { L"darr", 0x2193 },
-    { L"harr", 0x2194 },
-    { L"crarr", 0x21B5 },
-    { L"lArr", 0x21D0 },
-    { L"uArr", 0x21D1 },
-    { L"rArr", 0x21D2 },
-    { L"dArr", 0x21D3 },
-    { L"hArr", 0x21D4 },
-    { L"forall", 0x2200 },
-    { L"part", 0x2202 },
-    { L"exist", 0x2203 },
-    { L"empty", 0x2205 },
-    { L"nabla", 0x2207 },
-    { L"isin", 0x2208 },
-    { L"notin", 0x2209 },
-    { L"ni", 0x220B },
-    { L"prod", 0x220F },
-    { L"sum", 0x2211 },
-    { L"minus", 0x2212 },
-    { L"lowast", 0x2217 },
-    { L"radic", 0x221A },
-    { L"prop", 0x221D },
-    { L"infin", 0x221E },
-    { L"ang", 0x2220 },
-    { L"and", 0x2227 },
-    { L"or", 0x2228 },
-    { L"cap", 0x2229 },
-    { L"cup", 0x222A },
-    { L"int", 0x222B },
-    { L"there4", 0x2234 },
-    { L"sim", 0x223C },
-    { L"cong", 0x2245 },
-    { L"asymp", 0x2248 },
-    { L"ne", 0x2260 },
-    { L"equiv", 0x2261 },
-    { L"le", 0x2264 },
-    { L"ge", 0x2265 },
-    { L"sub", 0x2282 },
-    { L"sup", 0x2283 },
-    { L"nsub", 0x2284 },
-    { L"sube", 0x2286 },
-    { L"supe", 0x2287 },
-    { L"oplus", 0x2295 },
-    { L"otimes", 0x2297 },
-    { L"perp", 0x22A5 },
-    { L"sdot", 0x22C5 },
-    { L"lceil", 0x2308 },
-    { L"rceil", 0x2309 },
-    { L"lfloor", 0x230A },
-    { L"rfloor", 0x230B },
-    { L"lang", 0x2329 },
-    { L"rang", 0x232A },
-    { L"loz", 0x25CA },
-    { L"spades", 0x2660 },
-    { L"clubs", 0x2663 },
-    { L"hearts", 0x2665 },
-    { L"diams", 0x2666 }
+    { L"gt", 0x003E }
   };
 
   void printEntity(std::basic_ostream<Char>& os, const wchar_t* p)
@@ -662,7 +416,7 @@ void EntityResolver::getEntity(std::basic_ostream<Char>& os, Char ch) const
     else if (ch.value() >= ' ' && ch.value() <= 0x7F)
         os << ch;
     else
-        os << Char('&') << Char('#') << ch.value() << Char(';');
+        os << Char('&') << Char('#') << static_cast<uint32_t>(ch.value()) << Char(';');
 }
 
 
