@@ -98,7 +98,8 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             registerMethod("Map", *this, &BinRpcTest::Map);
             registerMethod("Multimap", *this, &BinRpcTest::Multimap);
             registerMethod("UnknownMethod", *this, &BinRpcTest::UnknownMethod);
-            registerMethod("CallPrefix", *this, &BinRpcTest::CallPrefix);
+            registerMethod("Domain", *this, &BinRpcTest::Domain);
+            registerMethod("WrongDomain", *this, &BinRpcTest::WrongDomain);
             registerMethod("Fault", *this, &BinRpcTest::Fault);
             registerMethod("Exception", *this, &BinRpcTest::Exception);
             registerMethod("CallbackException", *this, &BinRpcTest::CallbackException);
@@ -124,7 +125,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             connect(_loop.timeout, *this, &BinRpcTest::failTest);
             connect(_loop.timeout, _loop, &cxxtools::EventLoop::exit);
 
-            _server = new cxxtools::bin::RpcServer(_loop, "", _port);
+            _server = new cxxtools::bin::RpcServer(_loop, _port);
             _server->minThreads(1);
         }
 
@@ -142,18 +143,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onNothingFinished );
 
             multiply.begin();
-
-            _loop.run();
-        }
-
-        void onNothingFinished(const cxxtools::RemoteResult<bool>& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), false);
-
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(multiply.end(2000), false);
         }
 
         bool multiplyNothing()
@@ -227,18 +219,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<bool, bool, bool> multiply(client, "boolean");
-            connect( multiply.finished, *this, &BinRpcTest::onBooleanFinished );
 
             multiply.begin(true, true);
-
-            _loop.run();
-        }
-
-        void onBooleanFinished(const cxxtools::RemoteResult<bool>& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), true);
-
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(multiply.end(2000), true);
         }
 
         bool boolean(bool a, bool b)
@@ -257,18 +240,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onIntegerFinished );
 
             multiply.begin(2, 3);
-
-            _loop.run();
-        }
-
-        void onIntegerFinished(const cxxtools::RemoteResult<int>& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6);
-
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(multiply.end(2000), 6);
         }
 
         int multiplyInt(int a, int b)
@@ -285,18 +259,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<double, double, double> multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onDoubleFinished );
 
             multiply.begin(2.0, 3.0);
-
-            _loop.run();
-        }
-
-        void onDoubleFinished(const cxxtools::RemoteResult<double>& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6.0);
-
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(multiply.end(2000), 6.0);
         }
 
         double multiplyDouble(double a, double b)
@@ -313,18 +278,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<std::string, std::string> echo(client, "echoString");
-            connect( echo.finished, *this, &BinRpcTest::onStringEchoFinished );
 
             echo.begin("\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
-
-            _loop.run();
-        }
-
-        void onStringEchoFinished(const cxxtools::RemoteResult<std::string>& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
-
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(echo.end(2000), "\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
         }
 
         std::string echoString(std::string a)
@@ -341,17 +297,9 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<std::string, std::string, std::string> multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onEmptyFinished );
 
             multiply.begin("", "");
-
-            _loop.run();
-        }
-
-        void onEmptyFinished(const cxxtools::RemoteResult<std::string>& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "4");
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(multiply.end(2000), "4");
         }
 
         std::string multiplyEmpty(std::string a, std::string b)
@@ -370,15 +318,16 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onArrayFinished );
 
             std::vector<int> vec;
             vec.push_back(10);
             vec.push_back(20);
 
             multiply.begin(vec, vec);
-
-            _loop.run();
+            std::vector<int> r = multiply.end(2000);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.size(), 2);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.at(0), 100);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.at(1), 400);
         }
 
         std::vector<int> multiplyVector(const std::vector<int>& a, const std::vector<int>& b)
@@ -393,15 +342,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             return r;
         }
 
-        void onArrayFinished(const cxxtools::RemoteResult<std::vector<int> >& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 2);
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().at(0), 100);
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().at(1), 400);
-
-            _loop.exit();
-        }
-
         ////////////////////////////////////////////////////////////
         // EmptyArray
         //
@@ -411,19 +351,10 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onEmptyArrayFinished );
 
             std::vector<int> vec;
             multiply.begin(vec, vec);
-
-            _loop.run();
-        }
-
-        void onEmptyArrayFinished(const cxxtools::RemoteResult<std::vector<int> >& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 0);
-
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(multiply.end(2000).size(), 0);
         }
 
         ////////////////////////////////////////////////////////////
@@ -435,7 +366,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure< Color, Color, Color > multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onStuctFinished );
 
             Color a;
             a.red = 2;
@@ -448,17 +378,10 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             b.blue = 5;
 
             multiply.begin(a, b);
-
-            _loop.run();
-        }
-
-        void onStuctFinished(const cxxtools::RemoteResult<Color>& color)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().red, 6);
-            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().green, 12);
-            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().blue, 20);
-
-            _loop.exit();
+            Color r = multiply.end(2000);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.red, 6);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.green, 12);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.blue, 20);
         }
 
         Color multiplyColor(const Color& a, const Color& b)
@@ -479,7 +402,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<IntSet, IntSet, int> multiply(client, "multiplyset");
-            connect( multiply.finished, *this, &BinRpcTest::onSetFinished );
 
             IntSet myset;
             myset.insert(4);
@@ -488,19 +410,11 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             myset.insert(5);
 
             multiply.begin(myset, 2);
-
-            _loop.run();
-        }
-
-        void onSetFinished(const cxxtools::RemoteResult<IntSet>& result)
-        {
-            const IntSet& v = result.get();
+            const IntSet& v = multiply.end(2000);
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 3);
             CXXTOOLS_UNIT_ASSERT(v.find(8) != v.end());
             CXXTOOLS_UNIT_ASSERT(v.find(10) != v.end());
             CXXTOOLS_UNIT_ASSERT(v.find(22) != v.end());
-
-            _loop.exit();
         }
 
         IntSet multiplySet(const IntSet& s, int f)
@@ -520,7 +434,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<IntMultiset, IntMultiset, int> multiply(client, "multiplyset");
-            connect( multiply.finished, *this, &BinRpcTest::onMultisetFinished );
 
             IntMultiset myset;
             myset.insert(4);
@@ -529,19 +442,11 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             myset.insert(5);
 
             multiply.begin(myset, 2);
-
-            _loop.run();
-        }
-
-        void onMultisetFinished(const cxxtools::RemoteResult<IntMultiset>& result)
-        {
-            const IntMultiset& v = result.get();
+            const IntMultiset& v = multiply.end(2000);
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 4);
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.count(8), 1);
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.count(10), 2);
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.count(22), 1);
-
-            _loop.exit();
         }
 
         IntMultiset multiplyMultiset(const IntMultiset& s, int f)
@@ -561,7 +466,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<IntMap, IntMap, int> multiply(client, "multiplymap");
-            connect( multiply.finished, *this, &BinRpcTest::onMultiplyMapFinished );
 
             IntMap mymap;
             mymap[2] = 4;
@@ -569,13 +473,8 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             mymap[1] = -1;
 
             multiply.begin(mymap, 2);
+            const IntMap& v = multiply.end(2000);
 
-            _loop.run();
-        }
-
-        void onMultiplyMapFinished(const cxxtools::RemoteResult<IntMap>& result)
-        {
-            const IntMap& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 3);
             CXXTOOLS_UNIT_ASSERT(v.find(2) != v.end());
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.find(2)->second, 8);
@@ -583,8 +482,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.find(7)->second, 14);
             CXXTOOLS_UNIT_ASSERT(v.find(1) != v.end());
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.find(1)->second, -2);
-
-            _loop.exit();
         }
 
         IntMap multiplyMap(const IntMap& m, int f)
@@ -607,7 +504,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<IntMultimap, IntMultimap, int> multiply(client, "multiplymultimap");
-            connect( multiply.finished, *this, &BinRpcTest::onMultiplyMultimapFinished );
 
             IntMultimap mymap;
             mymap.insert(IntMultimap::value_type(2, 4));
@@ -616,13 +512,8 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             mymap.insert(IntMultimap::value_type(1, -1));
 
             multiply.begin(mymap, 2);
+            const IntMultimap& v = multiply.end(200);
 
-            _loop.run();
-        }
-
-        void onMultiplyMultimapFinished(const cxxtools::RemoteResult<IntMultimap>& result)
-        {
-            const IntMultimap& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 4);
             CXXTOOLS_UNIT_ASSERT(v.lower_bound(2) != v.end());
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.lower_bound(2)->second, 8);
@@ -635,8 +526,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(it->second, 16);
             CXXTOOLS_UNIT_ASSERT(v.lower_bound(1) != v.end());
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.lower_bound(1)->second, -2);
-
-            _loop.exit();
         }
 
         IntMultimap multiplyMultimap(const IntMultimap& m, int f)
@@ -651,36 +540,48 @@ class BinRpcTest : public cxxtools::unit::TestSuite
         }
 
         ////////////////////////////////////////////////////////////
-        // CallPrefix
+        // Domain
         //
-        void CallPrefix()
+        void Domain()
         {
-            _server->registerMethod("somePrefix.multiply", *this, &BinRpcTest::multiplyInt);
+            cxxtools::ServiceRegistry registry;
+            registry.registerMethod("multiply", *this, &BinRpcTest::multiplyInt);
+            _server->addService("myDomain", registry);
 
-            cxxtools::bin::RpcClient client(_loop, "", _port);
-            client.prefix("somePrefix.");
+            cxxtools::bin::RpcClient client(_loop, "", _port, "myDomain");
             cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onIntegerFinished );
 
             multiply.begin(2, 3);
-
-            _loop.run();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(multiply.end(2000), 6);
         }
 
+        ////////////////////////////////////////////////////////////
+        // WrongDomain
+        //
+        void WrongDomain()
+        {
+            cxxtools::ServiceRegistry registry;
+            registry.registerMethod("multiply", *this, &BinRpcTest::multiplyInt);
+            _server->addService("myDomain", registry);
+
+            cxxtools::bin::RpcClient client(_loop, "", _port, "unknownDomain");
+            cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
+
+            multiply.begin(2, 3);
+            CXXTOOLS_UNIT_ASSERT_THROW(multiply.end(2000), cxxtools::RemoteException);
+        }
+
+        ////////////////////////////////////////////////////////////
+        // UnknownMethod
+        //
         void UnknownMethod()
         {
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<bool> unknownMethod(client, "unknownMethod");
-            connect( unknownMethod.finished, *this, &BinRpcTest::onUnknownFinished );
 
             unknownMethod.begin();
 
-            CXXTOOLS_UNIT_ASSERT_THROW(_loop.run(), cxxtools::RemoteException);
-        }
-
-        void onUnknownFinished(const cxxtools::RemoteResult<bool>& r)
-        {
-            _loop.exit();
+            CXXTOOLS_UNIT_ASSERT_THROW(unknownMethod.end(2000), cxxtools::RemoteException);
         }
 
         ////////////////////////////////////////////////////////////
@@ -692,17 +593,11 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onFault );
             multiply.begin();
 
-            _loop.run();
-        }
-
-        void onFault(const cxxtools::RemoteResult<bool>& result)
-        {
             try
             {
-                result.get();
+                multiply.end(2000);
                 CXXTOOLS_UNIT_ASSERT_MSG(false, "cxxtools::RemoteException exception expected");
             }
             catch (const cxxtools::RemoteException& e)
@@ -710,8 +605,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
                 CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 7);
                 CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Fault");
             }
-
-            _loop.exit();
         }
 
         bool throwFault()
@@ -729,25 +622,19 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
-            connect( multiply.finished, *this, &BinRpcTest::onException );
+
             multiply.begin();
 
-            _loop.run();
-        }
-
-        void onException(const cxxtools::RemoteResult<bool>& result)
-        {
             try
             {
-                CXXTOOLS_UNIT_ASSERT_THROW(result.get(), cxxtools::RemoteException);
+                multiply.end(2000);
+                CXXTOOLS_UNIT_ASSERT(false);
             }
             catch (const cxxtools::RemoteException& e)
             {
                 CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 0);
                 CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Exception");
             }
-
-            _loop.exit();
         }
 
         bool throwException()
@@ -767,7 +654,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             cxxtools::bin::RpcClient client(_loop, "", _port);
             cxxtools::RemoteProcedure<unsigned, std::vector<int> > countSize(client, "countSize");
-            connect( countSize.finished, *this, &BinRpcTest::onCountSizeFinished );
 
             std::vector<int> v;
             v.resize(5000);
@@ -776,7 +662,7 @@ class BinRpcTest : public cxxtools::unit::TestSuite
 
             try
             {
-                _loop.run();
+                countSize.end(2000);
             }
             catch (const std::exception& e)
             {
@@ -785,12 +671,6 @@ class BinRpcTest : public cxxtools::unit::TestSuite
             }
         }
 
-        void onCountSizeFinished(const cxxtools::RemoteResult<unsigned>& r)
-        {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 5000);
-            _loop.exit();
-        }
-
         unsigned countSize(const std::vector<int>& v)
         {
             return v.size();
